<?php

    /**
     * This is a helper class designed to make GET and POST 
     * requests to the Media Portal API. 
     * */
    class APICommunicatorPHP {
        private $keyId;
        private $keySecret;
        
        /**
         * Initialize the key id and secret that will 
         * be used by this communicator.
         * 
         * @param string $key_Id        The ID of the API Key you'll be using to validate your API calls 
         * @param string $key_Secret    The Secret of the API Key you'll be using to validate your API calls 
         * */
        function APICommunicatorPHP($key_Id, $key_Secret){
            global $keyId, $keySecret;
            $keyId = $key_Id;
            $keySecret = $key_Secret;
        }
        
        /**
         * Computes the encoded authentication signature to be appended to a Media 
         * Portal API call. API Signatures are encoded in base64.
         * @param string $dateStr       Date in "D, d M Y H:i:s T" format
         * @param string $uri           Full URI of the service you whish to call (Not Including Parameters)
         * @param string $contentType   (Optional) typically "application/xml" or "application/json"
         * @param string $method        Typically "GET" or "POST"
         * @param string $md5           The MD5 Hashed value of your post body (send an empty string if for a GET request or post body is null)
         * @param string $key_secret    Secret associated with API Key
         * 
         * @return A string encoded in base64 to be added to an API call for authentication.
         * */
        function getAPISignature($dateStr, $uri, $contentType, $method, $md5, $keySecret){
            //The authentication signature is built as shown
            /* * * * * * * * * * * * * * * * * * * * * * * * *\ 
             * Mon, 01 Jan 2018 00:00:00 GMT\n               *
             * /api/uri/{access group ID}/{scid}/{property}\n*
             * application/json\n                            *
             * GET\n                                         *
             * MD5 Hash (if applicable)                      *
            \* * * * * * * * * * * * * * * * * * * * * * * * */
            $data = $dateStr . "\n" . $uri . "\n" . $contentType . "\n" . $method . "\n" . $md5;
            echo "<b>API Signature Pre-Hash:</b>" .$data . '<br/>';
                    
            //Compute the hash via the hash_mac function (requires PHP 5 >= 5.1.2, PECL hash >= 1.1)
            $hmac = hash_hmac("sha1", $data, $keySecret, true);
            
            //The hash to be sent in the request must be encoded in base64
            return base64_encode($hmac);
        }
      
        /**
         * This is the example code for making a GET service call from the Media Portal API.
         * This demo includes information on how requests are built and sent, and includes
         * information on creating authentication signatures (required in order to prove the
         * user has neccesarry permissions to access the results.
         * 
         * @param string $accept        The format for which data from the service call should be returned. Typically "application/xml" or "application/json."
         * @param string $dateStr       Date in "D, d M Y H:i:s T" format
         * @param string $url           Full URL of the service you whish to call (EX. https://fakeApiURL.com/services/key?param1=test1&param2=test2,test3)
         * */
        function sendTestAPICallGET($accept, $dateStr, $url){
            global $keyId, $keySecret;
            
            //First, create the api signature
            $signature = self::getAPISignature($dateStr, parse_url($url, PHP_URL_PATH), "", "GET", "", $keySecret);
            
            //Create the string to be set as the authorization property in the request header.
            //Format: MPA {key_id}:{signature}
            $authString = "MPA " . $keyId . ":" . $signature;
            
            //Create Header Properties Array.
            //Because this is a get, content-length == 0 and the properties content-type and content-md5 are not set.
            $headerProperties = array(
                'Accept: ' . $accept,
                'Authorization: ' . $authString,
                'Content-Length: 0',
                'Content-MD5: ',
                'Content-Type: ',
                'Date: ' . $dateStr
            );
    
            //Initialize curl request, set appropriate properties, and execute the request.
            $ch = curl_init();
            curl_setopt($ch, CURLOPT_URL, $url);
            curl_setopt($ch, CURLOPT_HTTPHEADER, $headerProperties );
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
            $result = curl_exec ($ch);
            curl_close ($ch);
            
            //Printing information of get request to screen
            echo "<b>Authentication property:</b>" . $authString . '<br/>';
            echo '<br/><b>Header Properties------</b><pre>'; 
            print_r($headerProperties); 
            echo '</pre><b>------------------</b><br/><br/>';
            echo '<br/><b>Results------</b><pre>';
            print_r($result);
            echo '</pre><b>------------------</b><br/><br/>';
          
        }
        
        /**
         * This is the example code for making a POST service call from the Media Portal API.
         * This demo includes information on how requests are built and sent, and includes
         * information on creating authentication signatures (required in order to prove the
         * user has neccesarry permissions to access the results.
         *
         * @param string $accept        The format for which data from the service call should be returned to the client. Typically "application/xml" or "application/json."
         * @param string $contentType   The format for which POST body parameters will be sent from the client. Typically "application/xml" or "application/json."
         * @param string $dateStr       Date in "D, d M Y H:i:s T" format
         * @param string $postBody      A string containing the JSON/XML POST request body parameter to be sent from the client.
         * @param string $url           Full URL of the service you whish to call (EX. https://fakeApiURL.com/services/key?param1=test1&param2=test2,test3)
         * */
        function sendTestAPICallPOST($accept, $contentType, $dateStr, $postBody, $url){
            global $keyId, $keySecret;
            
            //First, if you have a post body you will be sending with the request, create an MD5 hash of this post body.
            $md5 = "";
            if($postBody != null)
                $md5 = md5($postBody);
            
            //Create the api signature
            $signature = self::getAPISignature($dateStr, parse_url($url, PHP_URL_PATH), $contentType, "POST", $md5, $keySecret);
            
            //Create the string to be set as the authorization property in the request header.
            //Format: MPA {key_id}:{signature}
            $authString = "MPA " . $keyId . ":" . $signature;
            
            //Create Header Properties Array.
            //Because this is a post, content-type is set to reflect format of sent parameters (application/json, application/xml, etc.)
            $headerProperties = array(
                'Accept: ' . $accept,
                'Authorization: ' . $authString,
                'Content-Type: ' . $contentType,
                'Date: ' . $dateStr
            );
            
            //Initialize curl request, set appropriate properties, and execute the request.
            $ch = curl_init();
            if($postBody != null){
                //If you have a post body to send with the request...
                //Set Content-Length header property to the length of your post body
                array_push($headerProperties, 'Content-Length: '. strlen($postBody) );
                
                //Set Content-MD5 header property to the MD5 Hash of your post body
                array_push($headerProperties, 'Content-MD5: ' . $md5);
                
                //Set curl request property CURLOPT_POSTFIELDS to equal your post body
                curl_setopt( $ch, CURLOPT_POSTFIELDS, $postBody );
                
                
            } else {
                //Otherwise, set Content-MD5 header property to blank as you have no post body
                array_push($headerProperties, 'Content-MD5: ');
            }
            curl_setopt($ch, CURLOPT_URL, $url);
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
            curl_setopt($ch, CURLOPT_HTTPHEADER, $headerProperties );
            $result = curl_exec ($ch);
            curl_close ($ch);
            
            //Printing information of post request to screen
            echo "<b>Authentication property:</b>" . $authString . '<br/>';
            echo '<br/><b>Header Properties------</b><pre>';
            print_r($headerProperties);
            echo '</pre><b>------------------</b><br/><br/>';
            echo '<br/><b>Results------</b><pre>';
            print_r($result);
            echo '</pre><b>------------------</b><br/><br/>';
            
        }
     }
?>